@@ -2,7 +2,7 @@
  *                                                                   
  *                   Searduino
  *                      
- *   Copyright (C) 2011, 2012, 2013 Henrik Sandklef 
+ *   Copyright (C) 2011, 2012 Henrik Sandklef 
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
@@ -26,7 +26,6 @@
 #include "utils/print.h"
 
 #include "searduino_pin.h"
-#include "searduino_internal_log.h"
 #include "setup.h"
 #include "error.h"
 #include "ext_io.h"
@@ -64,14 +63,11 @@ FILE* proto_stream ;
  */
 static uint8_t stub_output_enabled = 1;
 
-#define RETURN_IF_HALTED() if (searduino_is_halted()) { printf ("CODE HALTED\n"); return 0; }  
 
 
 uint8_t
-ext_set_generic_input(uint8_t pin, unsigned int val, uint8_t pin_type)
+ext_set_generic_input(uint8_t pin, uint8_t val, uint8_t pin_type)
 {
-  //  printf ("%s (%d,%d)\n", __func__, pin, val);
-  LOG_PIN_VAL_FUNCTION(pin, val);
   input_callback(pin, val, pin_type);
 
   DEBUG_INFO(("ext_set_generic_input  %d,%d,%d",pin,val, pin_type));
@@ -83,7 +79,6 @@ int
 ext_get_generic_output(uint8_t pin, uint8_t pin_type)
 {
   DEBUG_INFO(("%d,%d",pin, pin_type));
-  LOG_PIN_FUNCTION(pin);
   return output_callback(pin, pin_type);
 }
 
@@ -91,7 +86,6 @@ uint8_t
 ext_set_input(uint8_t pin, unsigned int val, uint8_t pin_type)
 {
   PRINT_FUNCTION_NAME(("%d,%d,%d",pin,val, pin_type));
-  LOG_PIN_VAL_FUNCTION(pin, val);
 
   input_callback(pin,val, pin_type);
 
@@ -104,7 +98,6 @@ ext_get_dig_output(uint8_t pin)
 {
   uint8_t val ;
   PRINT_FUNCTION_NAME(("%d",pin));
-  LOG_PIN_VAL_FUNCTION(pin, val);
 
   val = digout_callback(pin);
 
@@ -120,7 +113,6 @@ ext_get_ana_output(uint8_t pin)
 {
   unsigned int val ;
   PRINT_FUNCTION_NAME(("%d",pin));
-  LOG_PIN_VAL_FUNCTION(pin, val);
 
   val = anaout_callback(pin);
 
@@ -135,7 +127,6 @@ ext_get_dig_mode(uint8_t pin)
 {
   uint8_t mode ;
   PRINT_FUNCTION_NAME(("%d",pin));
-  LOG_PIN_FUNCTION(pin);
 
   mode = dig_mode_callback(pin);
 
@@ -149,8 +140,7 @@ ext_get_dig_mode(uint8_t pin)
 out_to_sim_callback_ptr out_sim_callback = NULL;
 dm_to_sim_callback_ptr  dm_sim_callback = NULL ;
 log_to_sim_callback_ptr log_sim_callback = NULL ;
-lcd_to_sim_callback_ptr lcd_sim_callback = NULL ;
-pintype_to_sim_callback_ptr  pintype_sim_callback = NULL;
+
 
 
 void 
@@ -182,7 +172,7 @@ uint8_t
 ext_register_dig_mode_sim_cb(dm_to_sim_callback_ptr cb)
 {
   PRINT_FUNCTION_NAME(("%d",(int)cb));
-  LOG_FUNCTION();
+
   if (cb==NULL)
     {
       return SEARD_SEARDUINO_NULL_CALLBACK;
@@ -192,42 +182,6 @@ ext_register_dig_mode_sim_cb(dm_to_sim_callback_ptr cb)
   return SEARD_SEARDUINO_OK;
 }
 
-uint8_t
-ext_register_pin_type_sim_cb(pintype_to_sim_callback_ptr cb)
-{
-  PRINT_FUNCTION_NAME(("%d",(int)cb));
-  LOG_FUNCTION();
-  if (cb==NULL)
-    {
-      return SEARD_SEARDUINO_NULL_CALLBACK;
-    }
-
-  //printf ("TYPE CB AT %p\n", cb); 
-  pintype_sim_callback = cb;
-
-  return SEARD_SEARDUINO_OK;
-}
-
-uint8_t
-ext_inform_pin_type(int pin, int type)
-{
-  PRINT_FUNCTION_NAME(("%d,%d",(int)pin, (int)type));
-  LOG_FUNCTION();
-  if (pintype_sim_callback==NULL)
-    {
-      //      printf ("EXTERNALLY INFORM no cb\n");
-      return SEARD_SEARDUINO_NULL_CALLBACK;
-    }
-
-  //  printf ("EXTERNALLY INFORM %d %d\n", pin, type);
-  pintype_sim_callback(pin, type);
-
-  return SEARD_SEARDUINO_OK;
-}
-
-
-
-
 uint8_t
 ext_register_log_cb(log_to_sim_callback_ptr cb)
 {
@@ -242,21 +196,6 @@ ext_register_log_cb(log_to_sim_callback_ptr cb)
   return SEARD_SEARDUINO_OK;
 }
 
-uint8_t
-ext_register_lcd_cb(lcd_to_sim_callback_ptr cb)
-{
-  PRINT_FUNCTION_NAME(("%d",(int)cb));
-
-  if (cb==NULL)
-    {
-      return SEARD_SEARDUINO_NULL_CALLBACK;
-    }
-  lcd_sim_callback = cb;
-  //  printf ("lcd_sim_callback: %ul\n", lcd_sim_callback);
-
-  return SEARD_SEARDUINO_OK;
-}
-
 uint8_t
 ext_register_pinout_sim_cb(out_to_sim_callback_ptr cb)
 {
@@ -293,21 +232,9 @@ searduino_disable_streamed_output(void)
 int 
 ext_digital_set_mode(uint8_t pin, uint8_t mode)
 {
-  RETURN_IF_HALTED();
-
   /* Make sure all is set up before continuing*/
   init_ext_io();
 
-  LOG_PIN_VAL_FUNCTION(pin, mode);
-
-  /* printf ("PIN %d has mode: %d\n", pin, get_generic_pin_mode(pin)); */
-
-  /* If no change, spare the listeners from being callbacked */
-  if ( get_generic_pin_mode(pin) == mode )
-    {
-      return SEARD_ARDUINO_OK;
-    }
-
   /* If output enabled, print info on pin/val to stream*/
   if ( stub_output_enabled ) 
     {
@@ -329,38 +256,11 @@ ext_digital_set_mode(uint8_t pin, uint8_t mode)
 }
 
 
-int 
-ext_analog_set_mode(uint8_t pin, uint8_t mode)
-{
-  /* Make sure all is set up before continuing*/
-  init_ext_io();
-
-  /* If output enabled, print info on pin/val to stream*/
-  if ( stub_output_enabled ) 
-    {
-      fprintf(proto_stream, "amode:%d:%d  (in stub)\n", pin, mode); 
-      fflush(proto_stream);
-    }
-
-  /*
-   * Call registered listener
-   *
-   */
-  if ( dm_sim_callback != NULL )
-    {
-      /* Ok, the function pointer is not NULL, 
-	 so let's call it */
-      dm_sim_callback(pin, mode);
-    }
-  return SEARD_ARDUINO_OK;
-}
 
 
 int 
-ext_generic_write_outpin(uint8_t pin, unsigned int val, uint8_t pin_type)
+ext_generic_write_outpin(uint8_t pin, uint8_t val, uint8_t pin_type)
 {
-  RETURN_IF_HALTED();
-
   /* Make sure all is set up before continuing*/
   init_ext_io();
 
@@ -381,8 +281,5 @@ ext_generic_write_outpin(uint8_t pin, unsigned int val, uint8_t pin_type)
 	 so let's call it */
       out_sim_callback(pin, val, pin_type);
     }
-
   return SEARD_ARDUINO_OK;
 }
-
-
