@@ -23,6 +23,7 @@
 
 #include "arduino/wiring_private.h"
 #include "arduino/pins_arduino.h"
+#include "arduino/searduino_pin.h"
 #include "arduino/setup.h"
 #include "../include/arduino/error.h"
 #include "utils/print.h"
@@ -31,34 +32,8 @@
 #include "utils/error.h"
 #include <sys/time.h>
 
-
-typedef struct arduino_pin
-{
-  uint8_t         mode;
-  unsigned int    val;
-  struct timeval  last_write;
-  struct timeval  last_actual_write;
-  int             discard_ctr;
-} arduino_pin;
-
-/* 
- *  Warning, there be dragons here
- *
- *  Having this static implies zeroing it, which 
- *  sets the mode to INPUT (which is default on Arduinos)
- *
- *  It is ugeested that this is moved to some kind of 
- *  init function later on
- *
- */ 
-static arduino_pin arduino_analog_pins[NR_OF_ANALOG_PINS];
-
-#define PIN_OUT_OF_RANGE(pin)          (pin<0 || pin> NR_OF_ANALOG_PINS)
-#define get_analog_pin_mode(pin)      (arduino_analog_pins[pin].mode) 
-#define set_analog_pin_mode(pin,mode)  arduino_analog_pins[pin].mode=mode; 
-
-#define get_analog_pin_val(pin)       (arduino_analog_pins[pin].val) 
-#define set_analog_pin_val(pin,val)    arduino_analog_pins[pin].val=(val%1024);
+#include "ext_io.h"
+#include "setup.h"
 
 
 uint8_t analog_reference = 0;
@@ -68,44 +43,25 @@ void analogReference(uint8_t mode)
   analog_reference = mode;
 }
 
-unsigned int 
+int 
 analogRead(uint8_t pin)
 {
+  /* printf ("analogRead(%d)\n", pin); */
   searduino_setup();
   if (PIN_OUT_OF_RANGE(pin))
     {
       SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
       return 0;
     }
-
+  
   /* printf ("wiring: analogRead(%d) => %d\n",  */
   /* 	  pin, get_analog_pin_val(pin)); */
+  /* printf ("analogRead(%d)=>%d\n", pin, get_analog_pin_val(pin) ); */
+  /* printf ("analogRead(%d)=>%d\n", pin, get_generic_pin_val(pin, SEARDUINO_PIN_TYPE_ANALOG)); */
+  
   return get_analog_pin_val(pin);
 }
 
-/*
- *
- * Callback used to update in_pins
- * This fun is use (callbacked by) comm layer
- *
- */
-void 
-anain_callback(uint8_t pin, unsigned int val)
-{
-  searduino_setup();
-  PRINT_FUNCTION_NAME(("%d,%d",pin,val));
-
-  if (PIN_OUT_OF_RANGE(pin))
-    {
-      SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
-      return ;
-    }
-
-  set_analog_pin_val(pin,val);
-
-  /* printf ("%s:%s   storing in[%d].val=%d  (%d)\n",__FILE__, __func__, pin, arduino_analog_pins[pin].val, val);  */
-  return;
-}
 
 
 /*
@@ -125,14 +81,18 @@ anaout_callback(uint8_t pin)
       SEARD_ERROR(SEARD_ARDUINO_OUT_OF_BOUND);
       return 0;
     }
+  
+  if ( get_analog_pin_mode(pin) != INPUT )
+    {
+      fprintf(stderr, 
+	      "Can't read from a pin (%d) with mode (%d) NOT set INPUT\n",
+	      pin, get_analog_pin_mode(pin));
+      log_error("Can't read from an OUTPUT pin");
+      SEARD_ERROR(SEARD_ARDUINO_WRONG_PIN_MODE);
+      return 0;
+    }      
+  
+  
 
   return get_analog_pin_val(pin);
 }
-
-void analogWrite(uint8_t pin, int val)
-{
-  set_analog_pin_val(pin,val);
-
-  comm_analog_write_outpin(pin,val);
-}
-
