@@ -102,10 +102,6 @@ log_warning(char* s, ...)
     {
       log_sim_callback(SEARDUINO_LOG_LEVEL_WARNING, buffer);
     }
-  else
-    {
-      fprintf (stderr, " WARNING: %s\n", buffer);
-    }
 }
 
 
@@ -123,26 +119,18 @@ serial_print_s(char* s, ...)
     {
       log_sim_callback(SEARDUINO_LOG_SERIAL, buffer);
     }
-  else
-    {
-      fprintf(stderr, "%s", buffer);
-    }
 }
 
 void 
 serial_print_c(char c )
 {
-  static char buffer[]= {0,0};
+  char buffer[10];
 
-  buffer[0]=c;
+  sprintf(buffer, "%d", c);
   if (log_sim_callback!=NULL)
     {
       log_sim_callback(SEARDUINO_LOG_SERIAL, buffer);
     }
-  else
-    {
-      fprintf(stderr, "Serial fallback printer: %s\n", buffer);
-    }
 }
 
 void 
@@ -150,16 +138,12 @@ serial_print_l(long l )
 {
   char buffer[10];
 
-  sprintf(buffer, "%u", (unsigned int)l);
+  sprintf(buffer, "%u", l);
 
   if (log_sim_callback!=NULL)
     {
       log_sim_callback(SEARDUINO_LOG_SERIAL, buffer);
     }
-  else
-    {
-      fprintf(stderr, "%s", buffer);
-    }
 }
 
 
@@ -175,7 +159,6 @@ extern "C" {
   {
     Serial.println(c);
   }  
-
 }
 
 
